package Alzabo::Runtime::Index;

use strict;
use vars qw($VERSION);

use Alzabo::Runtime;

use base qw(Alzabo::Index);

$VERSION = sprintf '%2d.%02d', q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/;

1;

__END__

=head1 NAME

Alzabo::Runtime::Index - Index objects

=head1 SYNOPSIS

  use Alzabo::Runtime::Index;

=for pod_merge DESCRIPTION

=head1 INHERITS FROM

C<Alzabo::Index>

=for pod_merge merged

=for pod_merge METHODS

=head1 AUTHOR

Dave Rolsky, <autarch@urth.org>

=cut
