
#newid { }
.blue { color: #0099ff; font-size: 18px; line-height: 18px; font-family: Verdana, Arial, Geneva, Swiss, SunSans-Regular; text-decoration: none }
.body  { color: black; font-size: 8px; line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.comingsoon { color: #0099ff; font-style: italic; font-size: 10px; font-family: Verdana, Arial, Geneva, sans-serif }
.11pxblack  { color: black; font-size: 11px; line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.projectblack { color: black; font-size: 12px; font-family: Verdana, Arial, Courier }
.copyblack     { color: black; font-size: 10px; line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.copyblue     { color: #003366; font-size: 10px;  line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.copybluebold     { color: #003366; font-size: 10px; font-weight:bold;  line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.copy12  { color: black; font-size: 12px; font-family: Verdana, Arial, Courier }
.arialblack { color: black; font-size: 10px; line-height: 12px; font-family: Arial, Verdana, Courier; text-decoration: none }
.frontblack  { color: black; font-size: 10px; line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.copygrey  { color: #666666; font-size: 12px; line-height: 18px; font-family: Verdana, Arial, Courier; text-decoration: none }
.emphasis { color: #2a2a2a; font-weight: bold; font-size: 12px; line-height: 26px; font-family: Verdana, Arial, Geneva, Swiss, SunSans-Regular; background-color: white; border: 0px gray }
.example  { color: #434343; font-size: 9px; line-height: 12px; font-family: Verdana, Arial, Geneva, Swiss, SunSans-Regular }
.nav { color: #ff6600; font-size: 10px; font-family: Verdana, Arial, Geneva, Swiss, SunSans-Regular }
.number { color: #ff6600; font-weight: 800; font-size: 30px; font-family: Verdana, Arial, Courier }
.number2 { color: #d4d4d4; font-weight: 800; font-size: 30px; font-family: Verdana, Arial, Courier }
.orangesmall { color: #ff6600; font-size: 12px; font-family: Verdana, Arial, Geneva, sans-serif }
.orange10px { color: #ff6600; font-size: 10px; font-family: Verdana, Arial, Geneva, sans-serif }
.orangetext { color: #f60; font-size: 14px; font-family: Verdana, Arial, Geneva, sans-serif }
.orangebig      { color: #ff6600; font: italic bolder 21px/22px Verdana, Arial, Verdana; text-align: center; word-spacing: 1px; vertical-align: 2px }
.required  { color: #ff6600; font-weight: 300; font-size: 11px; font-family: Arial, Verdana, Courier }
.requiredsm { color: #ff6600; font-weight: 300; font-size: 10px; font-family: Verdana, Arial, Courier }
.teal 10px { color: #0099cc; font-size: 10px; font-family: Verdana, Arial, Geneva }
.title { color: white; font-weight: bold; font-size: 11px; font-family: Verdana, Arial, Geneva, Swiss, SunSans-Regular }
.whiteout { color: white; font-weight: 300; font-size: 11px; font-family: Verdana, Arial, Courier }
a  { color: blue; text-decoration: underline }
a:active { color: #369; text-decoration: none }
h1 { color: white; font-size: 20px; line-height: 18px }
a:visited { color: purple; text-decoration: underline }
a:Hover { color: #ff6600; text-decoration: underline }
.price { color: #8b0000; font-size: 10px; font-family: Arial, Verdana, Courier }
.peach { background-color: #fcc66 }
.grey { background-color: #f5f5f5 }
.darkgrey { background-color: #dcdcdc }


.orange12 { color: #ff6600; font-size: 12px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.orange16bold { color: #ff6600; font-size: 16px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.orangeskinny { color: #ff6600; font-size: 12px; font-family: Verdana, Arial, Geneva, sans-serif }
.orange10bi { color: #ff6600; font-size: 10px; font-weight:bold; font-style: italic; font-family: Verdana, Arial, Geneva, sans-serif }
.orange10 { color: #ff6600; font-size: 10px; font-family: Verdana, Arial, Geneva, sans-serif }
.orange8 { color: #ff6600; font-size: 8px; font-family: Verdana, Arial, Geneva, sans-serif }
.orange10bold { color: #ff6600; font-size: 10px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.black10 { color: #000000; font-size: 11px; font-family: Verdana, Arial, Geneva, sans-serif }
.black12 { color: #000000; font-size: 12px; font-family: Verdana, Arial, Geneva, sans-serif }
.black12 { color: #000000; font-size: 14px; font-family: Verdana, Arial, Geneva, sans-serif }
.black10bold { color: #000000; font-size: 10px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.black12bold { color: #000000; font-size: 12px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.gray10bold { color: #666666; font-size: 10px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.blue10bold { color: #003366; font-size: 10px; font-weight:bold;  line-height: 12px; font-family: Verdana, Arial, Courier; text-decoration: none }
.green16bold { color: #006633; font-size: 16px; font-weight:bold;  line-height: 20px; font-family: Verdana, Arial, Courier; text-decoration: none }
.teal16bold { color: #0099cc; font-size: 16px; font-weight:bold;  line-height: 24px; font-family: Verdana, Arial, Courier; text-decoration: none }
.blue10 { color: #003366; font-size: 10px; font-family: Verdana, Arial, Courier; text-decoration: none }
.whitetitle10 { color: #ffffff; font-size: 11px; font-weight:bold; font-family: Helvetica, Arial, Geneva, sans-serif }
.whitetitle { color: #ffffff; font-size: 12px; font-weight:bold; font-family: Helvetica, Arial, Geneva, sans-serif }
.whitetitle14 { color: #ffffff; font-size: 14px; font-weight:bold; font-family: Verdana, Arial, Geneva, sans-serif }
.red24 { color: red; font-weight:bold; font-size: 18px; font-family: Verdana, Arial, Courier }
.red16 { color: red; font-weight: 800; font-size: 18px; font-family: Verdana, Arial, Courier }
.red12 { color: red; font-weight: 800; font-size: 14px; font-family: Verdana, Arial, Courier }
