use Test;

BEGIN { Test::plan tests => 1; }

use Alzabo::Create;
use Cwd;
use File::Path;

require 't/utils.pl';

my $dir = cwd;

rmtree( "$dir/schemas", $Test::Harness::verbose );
unlink 't/dbmsynctest.dbm';

ok(1);

exit unless defined $ENV{ALZABO_RDBMS_TESTS};

my $tests = eval $ENV{ALZABO_RDBMS_TESTS};

foreach (@$tests)
{
    no strict 'refs';
    eval { &{ "$_->{rdbms}_clean_schema" }(%$_); };
}
